package com.deltax.fluttersdk.deltaxmobilesdk;

import android.content.Context;
import android.net.Uri;
import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.deltax.tracker.DeltaXLibrary;
import com.deltax.tracker.DeltaXTracker;

import java.util.HashMap;
import java.util.Map;

/**
 * DeltaxmobilesdkPlugin
 */
public class DeltaxmobilesdkPlugin implements FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;

    private Context context;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "deltaxmobilesdk");
        channel.setMethodCallHandler(this);
        this.context = flutterPluginBinding.getApplicationContext();
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result rawResult) {
//    if (call.method.equals("getPlatformVersion")) {
//      result.success("Android " + android.os.Build.VERSION.RELEASE);
//    } else {
//      result.notImplemented();
//    }
        if (!(call.arguments instanceof Map)) {
            throw new IllegalArgumentException("Map argument expected");
        }
        final String xb = call.argument("xb");
        Result result = new MethodResultWrapper(rawResult);
        DeltaXTracker trackerInstance = DeltaXLibrary.getTrackerInstance(xb);
        boolean doNotEncode = false;
        switch (call.method) {
            case "initTrackerInstance":
                DeltaXLibrary.initTrackerInstance(xb, context);
                result.success(null);
                break;
            case "setCustomerPhoneNumber":
                final String phoneNumber = call.argument("phoneNumber");
                if (phoneNumber == null) {
                    return;
                }
                doNotEncode = false;
                if (call.argument("doNotEncode") != null) {
                    doNotEncode = Boolean.parseBoolean(call.argument("doNotEncode").toString());
                }
                trackerInstance.setCustomerPhoneNumber(phoneNumber, doNotEncode);
                result.success(null);
                break;
            case "setCustomerEmail":
                final String email = call.argument("email");
                if (email == null) {
                    return;
                }
                doNotEncode = false;
                if (call.argument("doNotEncode") != null) {
                    doNotEncode = Boolean.parseBoolean(call.argument("doNotEncode").toString());
                }
                trackerInstance.setCustomerEmail(email, doNotEncode);
                result.success(null);
                break;
            case "setCustomerId":
                final String customerId = call.argument("customerId");
                if (customerId == null) {
                    return;
                }
                trackerInstance.setCustomerId(customerId);
                result.success(null);
                break;
            case "registerDxLinkDomain":
                final String dxLinkDomain = call.argument("dxLinkDomain");
                if(dxLinkDomain == null){
                    return;
                }
                trackerInstance.registerDxLinkDomain(dxLinkDomain);
                result.success(null);
                break;
            case "trackEvent":
                final HashMap<String, String> values = call.argument("values");
                if(values == null){
                    return;
                }
                trackerInstance.trackEvent(values);
                result.success(null);
                break;
            case "trackClick":
                final String uriStr = call.argument("uri");
                if(uriStr == null){
                    return;
                }
                final Uri uri = Uri.parse(uriStr);
                trackerInstance.trackClick(uri);
                result.success(null);
                break;
            case "disableTracking":
                DeltaXLibrary.disableTracking(context);
                result.success(null);
                break;
            case "enableTracking":
                DeltaXLibrary.enableTracking(context);
                result.success(null);
                break;
            default:
                result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    private static class MethodResultWrapper implements Result {
        private Result methodResult;
        private Handler handler;

        MethodResultWrapper(Result result) {
            methodResult = result;
            handler = new Handler(Looper.getMainLooper());
        }

        @Override
        public void success(final Object result) {
            handler.post(
                    new Runnable() {
                        @Override
                        public void run() {
                            methodResult.success(result);
                        }
                    });
        }

        @Override
        public void error(
                final String errorCode, final String errorMessage, final Object errorDetails) {
            handler.post(
                    new Runnable() {
                        @Override
                        public void run() {
                            methodResult.error(errorCode, errorMessage, errorDetails);
                        }
                    });
        }

        @Override
        public void notImplemented() {
            handler.post(
                    new Runnable() {
                        @Override
                        public void run() {
                            methodResult.notImplemented();
                        }
                    });
        }
    }
}


